------------------------------------------------------------
revno: 10418
revision-id: squid3@treenet.co.nz-20111208111329-4p5ugr1bj8lxdd8i
parent: squid3@treenet.co.nz-20111208111132-1gotx3bqkyq20w0h
fixes bug(s): http://bugs.squid-cache.org/show_bug.cgi?id=3442
committer: Amos Jeffries <squid3@treenet.co.nz>
branch nick: SQUID_3_1
timestamp: Thu 2011-12-08 04:13:29 -0700
message:
  Bug 3442: assertion failed: external_acl.cc:908: ch->auth_user_request != NULL
  
  External ACL sometimes cannot find the credentials in ACL Checklist even
  if they are attached to the HTTPRequest object.
  
  This seems to happen when the checklist is created and the line match
  started before the credentials are known. The credentials validation
  updates the HTTP request state but is not aware of ACL checklists needing
  to be updated so it never happens.
  
  This patch:
  * locate the %LOGIN value from either place where credentials can be found,
  * updates the checklist if it was unset,
  * passes '-' to the helper if no credentials at all were given.
  
  Although the earlier logics forcing a lookup means this '-' case should
  not happen it might if the external ACL were processed in 'fast' check.
------------------------------------------------------------
# Bazaar merge directive format 2 (Bazaar 0.90)
# revision_id: squid3@treenet.co.nz-20111208111329-4p5ugr1bj8lxdd8i
# target_branch: http://bzr.squid-cache.org/bzr/squid3/branches\
#   /SQUID_3_1/
# testament_sha1: c22dec8f9d72f3dbbde84accbd66f86e455c448f
# timestamp: 2011-12-08 11:21:07 +0000
# source_branch: http://bzr.squid-cache.org/bzr/squid3/branches\
#   /SQUID_3_1
# base_revision_id: squid3@treenet.co.nz-20111208111132-\
#   1gotx3bqkyq20w0h
# 
# Begin patch
=== modified file 'src/external_acl.cc'
--- src/external_acl.cc	2011-12-02 08:28:17 +0000
+++ src/external_acl.cc	2011-12-08 11:13:29 +0000
@@ -838,8 +838,13 @@
         switch (format->type) {
 
         case _external_acl_format::EXT_ACL_LOGIN:
-            assert (ch->auth_user_request);
-            str = ch->auth_user_request->username();
+            // if this ACL line was the cause of credentials fetch
+            // they may not already be in the checklist
+            if (ch->auth_user_request == NULL && ch->request)
+                ch->auth_user_request = ch->request->auth_user_request;
+
+            if (ch->auth_user_request != NULL)
+                str = ch->auth_user_request->username();
             break;
 #if USE_IDENT
 

